<?php

namespace Myriad\Squire;

use GuzzleHttp\Client;
use Illuminate\Support\ServiceProvider;
use Myriad\Squire\Contracts\Squire;
use Illuminate\Contracts\Support\DeferrableProvider;

class SquireServiceProvider extends ServiceProvider implements DeferrableProvider
{

    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = true;

    /**
     * Perform post-registration booting of services.
     *
     * @return void
     */
    public function boot()
    {
        $this->publishes([
            __DIR__.'/../config/squire.php' => config_path('squire.php'),
        ]);
    }

    /**
     * Register the service provider.
     */
    public function register(): void
    {
        $this->mergeConfigFrom(__DIR__.'/../config/squire.php', 'squire');

        $this->registerSquire();

        $this->app->alias('squire', Squire::class);
        $this->app->alias('squire', SquireClient::class);
    }

    /**
     * Register the Squire instance
     */
    private function registerSquire()
    {
        $this->app->singleton('squire', function ($app) {
            $guzzle = new Client();
            $config = new SquireConfiguration(config('squire'), $app['cache.store'], $guzzle);

            return new SquireClient($config, $guzzle);
        });
    }

    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return [
            'squire',
            Squire::class,
            SquireClient::class,
        ];
    }
}
